from stream_recorder.core import common, config
import stream_recorder.recorder.recorder
import stream_recorder.checker.etc
import rich.table
import rich.box
import asyncio
import typing


async def render_channels() -> None:
    common.progress.update(
        task_id=common.tasks.channels,
        visible=True,
    )
    tasks: list[typing.Coroutine] = []
    for url in config.channels_list:
        if url in common.recording_dict:
            recorder: stream_recorder.recorder.recorder.Recorder = common.recording_dict[url]
            common.channels_dict[url] = recorder.msg
        else:
            common.channels_dict[url] = '[yellow]checking...'
            tasks.append(check_and_update(url))
    update_table()
    stream_recorder.checker.etc.render()
    await asyncio.gather(*tasks)
    common.progress.update(
        task_id=common.tasks.channels,
        visible=False,
    )


async def check_and_update(
    url: str,
) -> None:
    if await stream_recorder.checker.etc.is_live(url):
        recorder = stream_recorder.recorder.recorder.Recorder(url)
        common.recording_dict[url] = recorder
        common.channels_dict[url] = recorder.msg
        recorder.recodrd_in_background()
    else:
        common.channels_dict[url] = '[orange1]no stream'
    update_table()


def update_table() -> None:
    common.table = rich.table.Table(
        show_header=False,
        box=rich.box.MINIMAL,
    )
    completed: int = 0
    for channel, status in common.channels_dict.items():
        if status != '[yellow]checking...':
            completed += 1
        common.table.add_row(channel, status)
    common.progress.update(
        task_id=common.tasks.channels,
        completed=completed,
        refresh=True,
    )
    stream_recorder.checker.etc.render()




from pathlib import Path
import asyncio
import sys
sys.path.append(str(Path(__file__).parent.parent.resolve()))
import stream_recorder.checker.main
import stream_recorder.recorder.recorder


async def async_main():
    await stream_recorder.checker.main.main()


def main():
    asyncio.run(async_main())


if __name__ == '__main__':
    main()




from stream_recorder.core import common, config
import stream_recorder.checker.etc
import datetime
import asyncio


async def render_timeout() -> None:
    common.progress.update(
        task_id=common.tasks.timeout,
        visible=True,
    )
    now = datetime.datetime.now()
    end = now + datetime.timedelta(seconds=config.timeout)
    while True:
        now = datetime.datetime.now()
        if now > end:
            break
        delta = end - now
        completed = 10 - delta.total_seconds()
        common.progress.update(
            task_id=common.tasks.timeout,
            completed=completed,
            refresh=True,
        )
        stream_recorder.checker.etc.render()
        await asyncio.sleep(0.1)
    common.progress.update(
        task_id=common.tasks.timeout,
        completed=config.timeout,
        visible=False,
    )




from stream_recorder.core import common
import stream_recorder.checker.channels
import stream_recorder.checker.timeout
import rich.live


async def render_all() -> None:
    with rich.live.Live(
        auto_refresh=False,
    ) as common.live:
        while True:
            await stream_recorder.checker.channels.render_channels()
            await stream_recorder.checker.timeout.render_timeout()


async def main():
    await render_all()




from stream_recorder.core import config
import rich.progress
import rich.console
import rich.table
import rich.live
import logging
import typing


channels_dict: dict[str, str] = {}
recording_dict: dict[str, typing.Any] = {}
group: rich.console.Group
table: rich.table.Table
live: rich.live.Live
progress: rich.progress.Progress = rich.progress.Progress(
    rich.progress.TextColumn("[progress.description]{task.description}"),
    rich.progress.BarColumn(),
    rich.progress.MofNCompleteColumn(),
    auto_refresh=False,
)


class tasks:
    channels: rich.progress.TaskID = progress.add_task(
        description='checking channels',
        total=len(config.channels_list),
        visible=False,
    )
    timeout: rich.progress.TaskID = progress.add_task(
        description='timeout',
        total=config.timeout,
        visible=False,
    )


class no_output:
    logger = logging.getLogger('ignore')
    logger.setLevel(logging.CRITICAL)
    params = {
        'logger': logger
    }





from stream_recorder.core import common
import rich.console
import asyncio
import yt_dlp


async def is_live(url: str) -> bool:
    with yt_dlp.YoutubeDL(params=common.no_output.params) as ydl:
        try:
            await asyncio.to_thread(
                ydl.extract_info,
                url,
                False,
            )
        except yt_dlp.utils.YoutubeDLError:
            return False
        else:
            return True


def render() -> None:
    common.group = rich.console.Group(common.table, common.progress)
    common.live.update(
        renderable=common.group,
        refresh=True,
    )




from stream_recorder.core import config, common
from pathlib import Path
import yt_dlp.utils
import datetime
import logging
import yt_dlp
import asyncio


class Recorder:
    def __init__(
        self,
        url: str,
    ) -> None:
        self.url: str = url
        now: str = datetime.datetime.now().strftime('%Y.%m.%d_%H-%M-%S')
        url_replaced = url.replace('/', '_')
        name: str = f'{now}_{url_replaced}'
        self.dir: Path = config.data / name
        self.log: Path = self.dir / 'log.txt'
        self.dir.mkdir(
            parents=True,
            exist_ok=True,
        )
        self.msg: str = f'[green]recording {self.dir}'
        self.logger = logging.getLogger(name)
        self.logger.setLevel(logging.DEBUG)
        self.logger.addHandler(logging.FileHandler(self.log))
        self.params: dict = {
            'logger': self.logger,
            'paths': {'home': str(self.dir)},
            'quiet': True,
            'keepvideo': True,
        }
        self.task: asyncio.Task

    def recodrd_in_background(self) -> None:
        self.task = asyncio.create_task(
            coro=self.record_async(),
        )

    async def record_async(self) -> None:
        await asyncio.to_thread(
            self.record_sync
        )
        common.recording_dict.pop(self.url)
        del self

    def record_sync(self) -> None:
        try:
            self.live_from_start()
        except yt_dlp.utils.DownloadError:
            self.no_live_from_start()

    def live_from_start(self) -> None:
        params = self.params.copy()
        params['live_from_start'] = True
        with yt_dlp.YoutubeDL(params) as ydl:
            ydl.download(self.url)

    def no_live_from_start(self) -> None:
        params = self.params.copy()
        params['no_live_from_start'] = True
        with yt_dlp.YoutubeDL(params) as ydl:
            ydl.download(self.url)




from pathlib import Path
import json


default_channels_list: list['str'] = [
    'youtube.com/@jolygolf8269/live',
    'youtube.com/@IzzyLaif/live',
    'twitch.tv/jolygames',
    'twitch.tv/jolygolf',
    'twitch.tv/izzylaif',
]
timeout: int = 10
repo: Path = Path(__file__).parent.parent.parent.resolve()
app_data: Path = Path('/app/data')
data: Path
if app_data.exists():
    data: Path = app_data
else:
    data: Path = repo / 'data'
data.mkdir(
    parents=True,
    exist_ok=True,
)
config_json: Path = data / 'config.json'
if not config_json.exists():
    config_json.write_text(json.dumps(default_channels_list))
channels_list: list['str'] = json.loads(config_json.read_text())
assert isinstance(channels_list, list)

